package nl.dragontamerfred.YRPerks;

import com.google.gson.Gson;
import nl.dragontamerfred.YRPerks.Types.ClassData;
import nl.dragontamerfred.YRPerks.Types.Classs;
import nl.dragontamerfred.YRPerks.Types.Perk;
import nl.dragontamerfred.YRPerks.Types.SubClassData;

import java.io.File;
import java.sql.*;
import java.util.*;

public class PerkManager {

    private static HashMap<String, Classs> classes = new HashMap<>();
    private static HashMap<String, Perk> perks = new HashMap<>();

    private static String url;

    public static void registerClass(Classs classs) {
        classes.put(classs.id, classs);
        YRPerks.logger.info("Registered class: " + classs.id);
    }

    public static void registerPerk(Perk perk) {
        perks.put(perk.id, perk);
        YRPerks.plugin.getServer().getPluginManager().registerEvents(perk, YRPerks.plugin);
        YRPerks.logger.info("Registered perk: " + perk.id);
    }

    /*
    selectedClass:      uuid (U)| selectedClass
    unlockedClasses:    uuid    | unlockedClass
    classData:          uuid    | class          | classData
     */
    public static boolean playerHasPerkSelected(UUID uuid, String perk) {
        ClassData cd = getClassDataForPlayer(uuid, getSelectedClassForPlayer(uuid).id);
        return cd != null && cd.hasPerk(perk);
    }

    public static void setClassUnlockedForPlayer(UUID uuid, String classID, boolean unlocked) {
        if (unlocked)
            unlockClassForPlayer(uuid, classID);
        else
            lockClassForPlayer(uuid, classID);
    }

    public static void setSubclassUnlocedForPlayer(UUID uuid, String classID, int subclassID, boolean unlocked) {
        ClassData cd = getClassDataForPlayer(uuid, classID);
        if (subclassID == 1)
            cd.subClassOne.unlocked = unlocked;
        else
            cd.subClassTwo.unlocked = unlocked;
        setClassDataForPlayer(uuid, classID, cd);
    }

    public static void lockClassForPlayer(UUID uuid, String classID) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                stmt.execute("DELETE * FROM selectedClass WHERE uuid='" + uuid + "' AND unlockedClass='" + classID + "'");
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static boolean isClassUnlockedForPlayer(UUID uuid, String classID) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                ResultSet result = stmt.executeQuery("SELECT * FROM unlockedClasses WHERE uuid='" + uuid + "'");
                while (result.next()) {
                    if (result.getString("unlockedClass").equals(classID))
                        return true;
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /*
    selectedClass:      uuid (U)| selectedClass
    unlockedClasses:    uuid    | unlockedClass
    classData:          uuid    | class          | classData
     */
    public static void setPerkForSlot(UUID uuid, int slot, String perk) {
        // Save it to the database
        Classs classs = getSelectedClassForPlayer(uuid);
        ClassData cd = getClassDataForPlayer(uuid, classs.id);
        if (cd == null) {
            cd = new ClassData(
                    classs.id,
                    1,
                    0,
                    new SubClassData(
                            false,
                            0
                    ),
                    new SubClassData(
                            false,
                            0
                    ),
                    "none",
                    "none",
                    "none",
                    "none",
                    "none");
        }
        cd.setPerkBySlot(slot, perk);
        setClassDataForPlayer(uuid, classs.id, cd);
    }

    public static Set<String> getAllPerks() {
        return perks.keySet();
    }

    public static List<String> getAllPerksForPlayer(UUID uuid) {
        ClassData cd = getClassDataForPlayer(uuid, getSelectedClassForPlayer(uuid).id);
        LinkedList<String> up = new LinkedList<>();
        if (cd != null) {
            Classs c = classes.get(cd.id);

            for (int i = 0; i <= cd.level; i++)
                up.add(c.levels.get(i).reward.perk);

            if (cd.subClass == 1)
                for (int i = 1; i <= cd.subClassOne.level; i++)
                    up.add(c.subClassOne.levels.get(i).reward.perk);

            else if (cd.subClass == 2)
                for (int i = 1; i <= cd.subClassTwo.level; i++)
                    up.add(c.subClassTwo.levels.get(i).reward.perk);
        }
        return up;
    }

    public static Perk getPerkById(String perkID) {
        return perks.getOrDefault(perkID, perks.get("none"));
    }

    public static void init(File dbFile) {
        try {
            Class.forName("org.sqlite.JDBC");

            url = "jdbc:sqlite:" + dbFile.getAbsolutePath();
            if (!dbFile.getParentFile().exists())
                dbFile.getParentFile().mkdirs();
            if (!dbFile.exists()) {
                createTable();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    /*
    selectedClass:      uuid (U)| selectedClass
    unlockedClasses:    uuid    | unlockedClass
    classData:          uuid    | class          | classData
     */
    public static void createTable() {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                // SQL statement for creating a new table

                stmt.execute("CREATE TABLE IF NOT EXISTS selectedClass (\n"
                        + "	uuid TEXT PRIMARY KEY,\n"
                        + "	selectedClass TEXT NOT NULL \n"
                        + ");");
                stmt.execute("CREATE TABLE IF NOT EXISTS unlockedClasses (\n"
                        + "	uuid TEXT NOT NULL,\n"
                        + "	unlockedClass TEXT NOT NULL \n"
                        + ");");
                stmt.execute("CREATE TABLE IF NOT EXISTS classData (\n"
                        + "	uuid TEXT NOT NULL,\n"
                        + " class TEXT NOT NULL,\n"
                        + "	classData TEXT NOT NULL \n"
                        + ");");
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
    selectedClass:      uuid (U)| class
    unlockedClasses:    uuid    | class
    classData:          uuid    | class          | classData
     */
    public static Classs getSelectedClassForPlayer(UUID uuid) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                ResultSet result = stmt.executeQuery("SELECT * FROM selectedClass WHERE uuid='" + uuid + "'");
                result.next();
                return classes.getOrDefault(result.getString("class"), classes.get("none"));
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return classes.get("none");
    }

    public static void setSelectedClassForPlayer(UUID uuid, String classID) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                stmt.execute("DELETE * FROM selectedClass WHERE uuid='" + uuid + "'");
                stmt.execute("INSERT INTO selectedClass ('" + uuid + "','" + classID + "')");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        ClassData cd = getClassDataForPlayer(uuid, classID);

    }

    public static void setClassDataForPlayer(UUID uuid, String classID, ClassData cd) {
        classData d = new classData(
                cd.id,
                cd.level,
                cd.subClass,
                new subClassData(
                        cd.subClassOne.unlocked,
                        cd.subClassOne.level
                ),
                new subClassData(
                        cd.subClassTwo.unlocked,
                        cd.subClassTwo.level
                ),
                new selectedPerks(
                        cd.perkOne,
                        cd.perkTwo,
                        cd.perkThree,
                        cd.perkFour,
                        cd.perkFive
                )
        );
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                Gson gson = new Gson();
                stmt.execute("DELETE * FROM classData WHERE uuid='" + uuid + "' AND class='" + classID + "'");
                stmt.execute("INSERT INTO classData ( '" + uuid + "', '"
                        + classID +"', '"
                        + gson.toJson(d, classData.class) + "')");
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void unlockClassForPlayer(UUID uuid, String classID) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                stmt.execute("INSERT INTO unlockedClasses ( ' " + uuid + " ', ' " + classID + " ' )");
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    /*
    selectedClass:      uuid (U)| class
    unlockedClasses:    uuid    | class
    classData:          uuid    | class          | classData
     */

    public static void setClassLevelForPlayer(UUID uuid, String classID, int level) {
        setClassDataForPlayer(uuid, classID, getClassDataForPlayer(uuid, classID).setLevel(level));
    }

    public static void setSubclassLevelForPlayer(UUID uuid, String classID, int subClassID, int level) {
        ClassData cd = getClassDataForPlayer(uuid, classID);
        if (subClassID==1)
            cd.subClassOne.level = level;
        else
            cd.subClassTwo.level = level;
        setClassDataForPlayer(uuid, classID, cd);
    }

    /**
     * Get the class data for a specific class.
     * @param uuid The UUID of the player
     * @param classID the id of the class
     * @return a {@link ClassData} object if there is data found, null if no data was found.
     */
    public static ClassData getClassDataForPlayer(UUID uuid, String classID) {
        try {
            Class.forName("org.sqlite.JDBC");

            try (Connection conn = DriverManager.getConnection(url);
                 Statement stmt = conn.createStatement()) {
                ResultSet result = stmt.executeQuery("SELECT * FROM classData WHERE uuid='" + uuid + "' AND class='" + classID+ "'");
                if (result.next()) {
                    Gson gson = new Gson();
                    classData cd = gson.fromJson(result.getString("classData"), classData.class);
                    selectedPerks sp = cd.selectedPerks;
                    return new ClassData(
                            cd.id,
                            cd.level,
                            cd.subClass,
                            new SubClassData(cd.subClassOne.unlocked, cd.subClassOne.level),
                            new SubClassData(cd.subClassTwo.unlocked, cd.subClassTwo.level),
                            sp.one,
                            sp.two,
                            sp.three,
                            sp.four,
                            sp.five
                    );
                } else {
                    return new ClassData(
                            classID,
                            1,
                            0,
                            new SubClassData(
                                    false,
                                    0
                            ),
                            new SubClassData(
                                    false,
                                    0
                            ),
                            "none",
                            "none",
                            "none",
                            "none",
                            "none"
                    );
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    static class classData {
        // The id of the class
        public String id;
        // The level that the class is at
        public int level;
        // The subClass that was chosen 0=none, 1=first, 2=second
        public int subClass;
        // The first subClass's data.
        public subClassData subClassOne;
        // The second subClass's data.
        public subClassData subClassTwo;
        // The selected perks
        public selectedPerks selectedPerks;

        public classData(String id, int level, int subClass, subClassData subClassOne, subClassData subClassTwo, selectedPerks selectedPerks) {
            this.id = id;
            this.level = level;
            this.subClass = subClass;
            this.subClassOne = subClassOne;
            this.subClassTwo = subClassTwo;
            this.selectedPerks = selectedPerks;
        }
    }

    static class subClassData {
        public boolean unlocked;
        public int level;

        public subClassData(boolean unlocked, int level) {
            this.unlocked = unlocked;
            this.level = level;
        }
    }

    static class selectedPerks {
        public String one;
        public String two;
        public String three;
        public String four;
        public String five;

        public selectedPerks(String one, String two, String three, String four, String five) {
            this.one = one;
            this.two = two;
            this.three = three;
            this.four = four;
            this.five = five;
        }
    }
}
